/*
This is a game demo written by Wingo Sai-Keung Wong.
OGRE is employed for graphics rendering.
CEGUI is employed for GUI.
Date: Dec 2006 - Jan 2007
Email: wingo.wong@gmail.com
*/
#ifndef __WAGO_COVERAGE_MAP_H__
#define __WAGO_COVERAGE_MAP_H__

#include "Ogre.h"
#include "OgreStringConverter.h"
#include "OgreException.h"
#include "common.h"

using namespace Ogre;

using namespace std;

class WAGO_COVERAGE_MAP : public ManualResourceLoader {
protected:
	
private:
	int mNumChannels;
	uint mWidth, mHeight, mSize;
      uchar* mData;
      TexturePtr mTexture;
protected:
//	void loadResource(Resource*);
public:
	WAGO_COVERAGE_MAP(const String &name, const String &group, uint nx, uint nz, int numChannels);
	size_t getWidth() const { return mWidth;}
	size_t getHeight() const { return mHeight;}
	
	void loadMap(const String &filename, const String &res_group);
	uchar getValue(uint x, uint y, uint c) const
    {
      return mData[(y*mWidth + x)*mNumChannels + c];
    }

    void setValue(uint x, uint y, uint c, uchar val)
    {
      mData[(y*mWidth + x)*mNumChannels + c] = val;
    }
	void updateTexture()
    {
      // write the edit buffer into the texture's pixel buffer
      HardwarePixelBufferSharedPtr buffer = mTexture->getBuffer();
      PixelBox pixelBox (mWidth, mHeight, 1, getFormat(mNumChannels), mData);
      Image::Box imageBox (0, 0, mWidth, mHeight);
      buffer->blitFromMemory(pixelBox, imageBox);
    }
	   const String& getName() const
    {
      return mTexture->getName();
    }
	void loadFromImage(Image& image)
    {
		image.resize(mWidth, mHeight);
      if (image.getWidth() != mWidth || image.getHeight() != mHeight)
        OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Given image doesn't conform to the used width and height.", "CoverageMap::loadFromImage");
      if (image.getFormat() != getFormat(mNumChannels) && image.getFormat() != getFormat(-mNumChannels))
        OGRE_EXCEPT(Exception::ERR_INVALIDPARAMS, "Given image is of invalid pixel format.", "CoverageMap::loadFromImage");

      memcpy(mData, image.getData(), image.getSize());
      if (getChannels(image.getFormat()) <= -3)
      {
        // need RGB(A), but given BGR(A) so invert
        for (uint i = 0; i < mSize; i += mNumChannels)
          swap(mData[i], mData[i+2]);
      }

      updateTexture();
    }
public:
	int getNumChannelsPerCoverageMap() { return mNumChannels;}
	void saveToImage(Image& image)
    {
      image.loadDynamicImage(mData, mWidth, mHeight, 1, getFormat(mNumChannels));
    }
private:
	PixelFormat getFormat(int channels)
    {
      switch (channels)
      {
      case 1: return PF_BYTE_A;
      case 2: return PF_BYTE_LA;
      case 3: return PF_BYTE_RGB;
      case 4: return PF_BYTE_RGBA;
      case -1: return PF_BYTE_A;
      case -2: return PF_BYTE_LA;
      case -3: return PF_BYTE_BGR;
      case -4: return PF_BYTE_BGRA;
      default: return PF_UNKNOWN;
      }
    }

    int getChannels(PixelFormat format)
    {
      switch (format)
      {
      case PF_BYTE_A: return 1;
      case PF_BYTE_LA: return 2;
      case PF_BYTE_RGB: return 3;
      case PF_BYTE_BGR: return -3;
      case PF_BYTE_RGBA: return 4;
      case PF_BYTE_BGRA: return -4;
      default: return 0;
      }
    }
    void loadResource(Resource*)
    {
      // the texture has requested to (re)load, we just copy our edit buffer
      // into the texture
      mTexture->createInternalResources();
      updateTexture();
    }

    
	
	

    
};

#endif