/*
This is a game demo written by Wingo Sai-Keung Wong.
OGRE is employed for graphics rendering.
CEGUI is employed for GUI.
Date: Dec 2006 - Jan 2007
Email: wingo.wong@gmail.com
*/
#include "ExampleApplication.h"
#include "mymesh.h"
#include "coverage_map.h"

#define CAMERA_NAME "CAMERA_NAME"

class GameApplication: public ExampleApplication
{
public:
	GameApplication() {}
	//
	
	//
	void createCamera(void) {
		Camera *cam;
		cam = mSceneMgr->createCamera(CAMERA_NAME);
		cam->setPosition(Vector3(0,550,0));
		cam->lookAt(Vector3(0,0,1));
		cam->setNearClipDistance(5);
		mCamera = cam; // mCamera is passed to the frame listener. we have to update.
	}
	void createScene() {
		//
		createCoverageMap();
		//
		/*
		Entity *ent = mSceneMgr->createEntity("ogre", "ogrehead.mesh");
		ent->setCastShadows(true);
		mSceneMgr->getRootSceneNode()->createChildSceneNode()->attachObject(ent);
		*/
		Light *light = mSceneMgr->createLight("Light1");
		light->setType(Light::LT_POINT);
		light->setPosition(Vector3(0, 150, 250));

		light->setDiffuseColour(1.0, 0.0, 0.0);
		light->setSpecularColour(1.0, 0.0, 0.0);

		light = mSceneMgr->createLight("Light3");
		light->setType(Light::LT_DIRECTIONAL);
		light->setDiffuseColour(ColourValue(.25, .25, 0));
		light->setSpecularColour(ColourValue(.25, .25, 0));

		light->setDirection(Vector3( 0, -1, 1 )); 

		light = mSceneMgr->createLight("Light2");
		light->setType(Light::LT_SPOTLIGHT);
		light->setDiffuseColour(0, 0, 1.0);
		light->setSpecularColour(0, 0, 1.0);
		light->setDirection(-1, -1, 0);
		light->setPosition(Vector3(300, 300, 0));
		//
		MyMesh *mesh = new MyMesh(mSceneMgr);
		mesh->createMesh("ETTerrainMaterial");
	}
private:
	void createCoverageMap() {
		WAGO_COVERAGE_MAP *c;
		c = new WAGO_COVERAGE_MAP("ETSplatting0", "My", 128, 128, 3);
		c->loadMap("ETcoverage.0.png", "My");
		//
		c = new WAGO_COVERAGE_MAP("ETSplatting1", "My", 128, 128, 3);
		c->loadMap("ETcoverage.1.png", "My");
		//
		c = new WAGO_COVERAGE_MAP("ETLightmap", "My", 128, 128, 3);
		c->loadMap("ETlightmap.png", "My");
	}
};

namespace {
	GameApplication *app = new GameApplication;
};



int main(int argc, char **argv)
{
	try {
		app->go();
	} catch( Exception& e ) {

#if OGRE_PLATFORM == OGRE_PLATFORM_WIN32
		//MessageBox( NULL, e.getFullDescription().c_str(), TEXT("An exception has occured!"), MB_OK | MB_ICONERROR | MB_TASKMODAL);
		MessageBox( NULL, NULL, TEXT("An exception has occured!"), MB_OK | MB_ICONERROR | MB_TASKMODAL);
#else
		std::cerr << "An exception has occured: " << e.getFullDescription();
#endif

	}
}
