#include "mymesh.h"

#define CIRCLE_SIZE 10

MyMesh::MyMesh(SceneManager *m)
{
	mSceneMgr = m;
	w = 400;
	h = 400;
}

void MyMesh::buildMesh(HardwareVertexBufferSharedPtr posnormVertexBuffer)
{
	int numVertices = 8;
	int i;
	float *posnormBufData = (float*) posnormVertexBuffer->lock(HardwareBuffer::HBL_DISCARD);
	int j;
	Real y = 0;
	for(i=0;i<numVertices;i++) {
		if (i < 4) {
			j = i;
			y = -100+0;
		} else {
			j = i - 4;
			y = -100+0.5;
		}
		posnormBufData[6*i+0]=((Real)(j%2)-0.5f)*w; // pos X
		posnormBufData[6*i+2]=((Real)(j/2)-0.5f)*h; // pos Y
		posnormBufData[6*i+1]=y; // pos Z
		posnormBufData[6*i+3]=0 ; // normal X
		posnormBufData[6*i+5]=1 ; // normal Y
		posnormBufData[6*i+4]=0 ; // normal Z
	}
	posnormVertexBuffer->unlock();
}

void MyMesh::createMesh(const String &matName)
{
	int i,lvl ;

	String name;
	genNameUsingIndex("info", 0, name);
	//
	mMesh = MeshManager::getSingleton().createManual(name,
		ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME) ;
	SubMesh* subMesh = mMesh->createSubMesh();
	subMesh->useSharedVertices=false;

	int numVertices = 8 ;
	int indexCount = 12;
	bool first = true;
	if (first) { // first Circle, create some static common data
		first = false ;

		// static buffer for position and normals
		posnormVertexBuffer =
			HardwareBufferManager::getSingleton().createVertexBuffer(
			6*sizeof(float), // size of one vertex data
			numVertices, // number of vertices
			HardwareBuffer::HBU_STATIC_WRITE_ONLY, // usage
			false); // no shadow buffer

		buildMesh(posnormVertexBuffer);
		// static buffers for 16 sets of texture coordinates
		texcoordsVertexBuffers = new HardwareVertexBufferSharedPtr[16];
		for(lvl=0;lvl<16;lvl++) {
			texcoordsVertexBuffers[lvl] =
				HardwareBufferManager::getSingleton().createVertexBuffer(
				2*sizeof(float), // size of one vertex data
				numVertices, // number of vertices
				HardwareBuffer::HBU_STATIC_WRITE_ONLY, // usage
				false); // no shadow buffer
			float *texcoordsBufData = (float*) texcoordsVertexBuffers[lvl]->
				lock(HardwareBuffer::HBL_DISCARD);
			//float x0 = (Real)(lvl % 4) * 0.25 ;
			//float y0 = (Real)(lvl / 4) * 0.25 ;
			float x0 = 0;
			float y0 = 0;
			int j;
			//y0 = 0.75-y0 ; // upside down
			for(i=0;i<numVertices;i++) {

				if (i <4 ) {
					x0 = 0;
					y0 = 0;
					j = i;
				} else {
					x0 = 0;
					y0 = 0;
					j = i - 4;
				}
				texcoordsBufData[i*2 + 0]=
					x0 + 1.0 * (Real)(j%2) ;
				texcoordsBufData[i*2 + 1]=
					y0 + 1.0 * (Real)(j/2) ;
			}
			texcoordsVertexBuffers[lvl]->unlock();
		}

		// Index buffer for 4 faces

		unsigned short faces[12] = {2,1,0,  2,3,1,
			6,5,4,  6,7,5
		};
		//unsigned short faces[6] ={2,1,0,  2,3,1};
		indexBuffer =
			HardwareBufferManager::getSingleton().createIndexBuffer(
			HardwareIndexBuffer::IT_16BIT,
			indexCount,
			HardwareBuffer::HBU_STATIC_WRITE_ONLY);
		indexBuffer->writeData(0,
			indexBuffer->getSizeInBytes(),
			faces,
			true); // true?
	}

	// Initialize vertex data
	subMesh->vertexData = new VertexData();
	subMesh->vertexData->vertexStart = 0;
	subMesh->vertexData->vertexCount = numVertices;
	// first, set vertex buffer bindings
	VertexBufferBinding *vbind = subMesh->vertexData->vertexBufferBinding ;
	vbind->setBinding(0, posnormVertexBuffer);
	vbind->setBinding(1, texcoordsVertexBuffers[0]);
	// now, set vertex buffer declaration
	VertexDeclaration *vdecl = subMesh->vertexData->vertexDeclaration ;
	vdecl->addElement(0, 0, VET_FLOAT3, VES_POSITION);
	vdecl->addElement(0, 3*sizeof(float), VET_FLOAT3, VES_NORMAL);
	vdecl->addElement(1, 0, VET_FLOAT2, VES_TEXTURE_COORDINATES);

	// Initialize index data
	subMesh->indexData->indexBuffer = indexBuffer;
	subMesh->indexData->indexStart = 0;
	subMesh->indexData->indexCount = indexCount;

	// set mesh bounds
	AxisAlignedBox circleBounds(-CIRCLE_SIZE/2.0f, 0, -CIRCLE_SIZE/2.0f,
		CIRCLE_SIZE/2.0f, 0, CIRCLE_SIZE/2.0f);
	mMesh->_setBounds(circleBounds);
	mMesh->load();
	mMesh->touch();

	mNode = static_cast<SceneNode*> (mSceneMgr->getRootSceneNode()->createChild(name));

	mEntity = mSceneMgr->createEntity(name, name);
	mEntity->setMaterialName(matName);

	mNode->attachObject(mEntity);
}
